/***********************************************************************************************************
 * Une classe représentant un Feu de signalisation (feu rouge) basé sur 3 Led connectées sur des pin arduino
 * 
 *  IUT GEII TOULON - Correction du TP n° 3
 *  M.GRIMALDI - Octobre 2020
 */

 
#include "Feu.h"
#include <Arduino.h>


// Constructeur permettant d'initialiser un Feu à partir de trois objets Led - state=0(vert), durations=1000 (ms pour les trois)
Feu::Feu(Led vert, Led orange, Led rouge) {
    leds[0]=vert; leds[1]=orange ; leds[2]= rouge;
    mode=0;         // manuel
    for (int i=0 ; i<3 ; i++)durations[i]=1000;     // 1 seconde à tout le monde
    set_state(0);             //vert
    lastMillis = 0;
}

// setter: permet de modifier l'état (0=vert, 1=orange, 2=rouge)
void Feu::set_state(int value) {
    if (value>=0 && value<=2) {         // un état valide
        state = value;
        for (int i=0 ; i<3 ; i++)       // on met les leds dans cet état
            if (i==state)leds[i].lightOn();
                else leds[i].lightOff();
    }
}

// setter: permet de modifier le mode de fonctionnement courant (0=manuel, 1=automatique, 2=orange clignotant)
void Feu::set_mode(int value) {
    if (value>=0 && value <=2)  mode = value;
    if (mode == MODE_ORANGE_CLI) set_state(ORANGE);
}

// setter: permet de régler la durée d'une couleur
void Feu::set_duration(int couleur, int duration){
    if (couleur>=0 && couleur<=2)durations[couleur] = duration;
}

// setter: permet de régler la durée des trois couleurs: vert, orange, rouge
void Feu::set_duration(int vert, int orange, int rouge){
    durations[VERT]=vert;
    durations[ORANGE]=orange;
    durations[ROUGE]=rouge;
}


// getter: permet de récupérer l'état courant du Feu (0=vert, 1=orange, 2=rouge)
int Feu::get_state(){
  return state;
}

// getter: renvoie le mode de fonctionnement courant (0=manuel, 1=automatique, 2=orange clignotant)
int Feu::get_mode(){
  return mode;
}

// Cette méthode permet de synchroniser le feu, c'est à dire de modifier son état en fonction du mode est des durées d'allumage de chaque couleur.
// Pour que le Feu fonctionne automatiquement, il faut l'appeler dans la boucle principale du programme. La fonction sync() n'est pas bloquante.
void Feu::sync() {
    unsigned long now = millis();                   // l'instant courant
    
    switch(mode){
    case 0:     // manuel on ne fait rien
    case 1:     // automatique on change d'état si la durée est terminée
        if (now-lastMillis >= durations[state]){      // c'est fini pour l'état courant
            set_state((state+1)%3);                   // on passe à l'état suivant
            lastMillis = now;                         // on garde l'instant du dernier changement
        }
        break;
    case 2:     // orange clignotant
        if (now-lastMillis>=durations[ORANGE]){
                leds[ORANGE].invert();     // l'orange clignote
                lastMillis = now;
        }
        break;
    }
}
