/***********************************************************************************************************
 * Une classe représentant un Feu de signalisation (feu rouge) basé sur 3 Led connectées sur des pin arduino
 * 
 *  IUT GEII TOULON - Correction du TP n° 3
 *  M.GRIMALDI - Octobre 2020
 *  
 */
 
#ifndef _FEU_H
#define _FEU_H

#include "Led.h"

#define VERT 0
#define ORANGE 1
#define ROUGE 2

#define MODE_MANUEL 0
#define MODE_AUTO 1
#define MODE_ORANGE_CLI 2

//    Cette classe représente un feu de signalisation basé sur trois leds directement connectées sur des ports d'une carte arduino

class Feu {
  private:
    // les trois leds du feu de signalisation: //leds[0]=vert, leds[1]=orange, leds[2]=rouge
    Led leds[3];

    //l'état du feu: 0=vert, 1=orange, 2=rouge
    int state;

    // cet attribut est un tableau qui contiendra les durées d'allumage des différentes leds (0=vert, 1=orange, 2=rouge)
    int durations[3];

     // le mode de fonctionnement du Feu (0=manuel, 1=automatique, 2=orange clignotant)
    int mode;

    // l'instant du dernier changement d'état
    unsigned long lastMillis;


  public:
    // Constructeur permettant d'initialiser un Feu à partir de trois Led - state=0(vert), durations=1000 (ms pour les trois)
    Feu(Led vert, Led orange, Led rouge);

    // acesseurs, getter/setter
    int get_state();  
    void set_state(int value);   
    int get_mode();
    void set_mode(int value);

    // setter: permet de régler la durée d'une couleur
    void set_duration(int number, int duration);

    // setter: permet de régler la durée des trois couleurs vert, orange, rouge
    void set_duration(int vert, int orange, int rouge);


    // Cette méthode permet de synchroniser le feu avec la boucle principal. 
    // Pour que le Feu fonctionne automatiquement, il faut l'appeler dans la boucle principale du programme. La fonction sync n'est pas bloquante.
    void sync();

};

#endif
